OscSend send; //send OSC
OscRecv recv; //recv OSC
Server server; //instantiate the server

send.setHost( "127.0.0.1", 9000);
9001 => recv.port;
recv.listen(); // start listening (launch thread)



class OscListener { // general listener template
    OscEvent e;
                    
    fun void action(){} //to be extended

    fun void bind( string arg ){
            recv.event(arg) @=> e; // bind event to address
            spork ~ oscShred( ); // start listening to incomming osc
    }
                    
    fun void oscShred() {
            while (on){
                    e => now;
                    if ( e.nextMsg()){ action(); }
            }
            //<<<"exiting shred", me>>>;
            //machine.remove( s.id() );
    }
}

// special osc event listeners ///////////////////////////////
class StatusListener extends OscListener{
    fun void action(){
        if (e.getInt()){
            <<<"chuck: connected msg received from widgets server">>>;
            //send.startMsg( "/status", "i" );
            //1 => send.addInt; // send connected message to graphics
        } else {
            <<<"chuck: widgets server just quit">>>;
            0 => on;
        }
    }
}
// set window props //
class FrameListener extends OscListener{
    fun void action(){ setWindowProps(); }
}
// creating widgets on server //
class WidgetsListener extends OscListener{
    fun void action(){ doWidgets(); }
}


// instantiating general server event listeners
StatusListener statusLis;
statusLis.bind( "/status, i" );
FrameListener frameLis;
frameLis.bind( "/frame, i" );
WidgetsListener widgetsLis;
widgetsLis.bind( "/widgets, i" );


class Server {
    ///classes to interface with the widgets OSC commands////
    fun void setGUIProps(string name, int x, int y, int w, int h, int fps){
        send.startMsg( "/frame", "s i i i i i" );
        name => send.addString; //name
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
        fps => send.addInt; //fps
    }
    
    // widgets /////////////////////
    fun void doLabel(string label, int x, int y, int w, int h) {
        send.startMsg( "/label", "s i i i i" );
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
    }

    fun void doButton(string oscaddress, string label, int x, int y, int w, int h) {
        send.startMsg( "/button", "s s i i i i" );
        oscaddress => send.addString;
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
    }

    fun void doToggle(string oscaddress, string label, int x, int y, int w, int h) {
        send.startMsg( "/toggle", "s s i i i i" );
        oscaddress => send.addString;
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
    }

    fun void doSlider(string oscaddress, string label, int x, int y, int w, int h, float min, float max,
                      string style, float value) {
        send.startMsg( "/slider", "s s i i i i f f s f" );
        oscaddress => send.addString;
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
        min => send.addFloat; // min range
        max => send.addFloat; // max range
        style => send.addString; // style
        value => send.addFloat; // value initial
    }

    fun void doChoice(string oscaddress, string label, int x, int y, int w, int h, string choices) {
        send.startMsg( "/choice", "s s i i i i s" );
        oscaddress => send.addString;
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
        choices => send.addString; // choices as string containing a list like "['one', 'two']", or "[1,2,3]"
    }

    fun void doRadioBox(string oscaddress, string label, int x, int y, int w, int h, string choices, int selected) {
        send.startMsg( "/radiobox", "s s i i i i s i" );
        oscaddress => send.addString;
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
        choices => send.addString; // choices as string containing a list like "['one', 'two']", or "[1,2,3]"
        selected => send.addInt; // selected choice on initialisation
    }

    fun void doNumber(string oscaddress, string label, int x, int y, int w, int h, float min, float max, float value) {
        send.startMsg( "/number", "s s i i i i f f f" );
        oscaddress => send.addString;
        label => send.addString; 	   
        x => send.addInt; // x
        y => send.addInt; // y
        w => send.addInt; //width
        h => send.addInt; //height
        min => send.addFloat; // min range
        max => send.addFloat; // max range
        value => send.addFloat; // initial value
    }
}

send.startMsg( "/status", "i" );
1 => send.addInt; // send connected message to graphics

///////////////////////////////////////////////////
///////////////////////////////////////////////////
///////////////////////////////////////////////////
// ok now we can do the actual stuff
///////////////////////////////////////////////////



// GUI setup functions //////////////////////////////////
fun void setWindowProps() { // this function is called by the server at the right time
    <<<"chuck : passing frame props via osc">>>;
    // name, xloc, yloc, width, height, fps
    server.setGUIProps("gui test all widgets", 100, 100, 450, 350, 10);
}

fun void doWidgets() { // this function is called by the server at the right time
    <<<"chuck : creating widgets via osc">>>;

    // label --> args :  label, x, y, width, height, rangemin, rangemax, style
    server.doLabel("some text", 50, 100, 20, 20); //NOTE that width and height are not working yet in this widget

    // toggle --> args :  oscaddress, label, x, y, width, height
    server.doToggle("/toggle1", "just a toggle", 10, 10, 90, 50);

    // button --> args :  oscaddress, label, x, y, width, height
    server.doButton("/button1", "just a button", 120, 10, 100, 40);

    // slider --> args :  oscaddress, label, x, y, width, height, rangemin, rangemax, style
    server.doSlider("/slider1", "just a slider", 270, 10, 160, 40, 10, 400, "horizontal", 20);
    // slider2 vertical 
    server.doSlider("/slider2", "just a slider", 170, 100, 55, 140, 100, 4000, "vertical", 10);

    // number --> args :  oscaddress, label, x, y, width, height, rangemin, rangemax, initialvalue
    server.doNumber("/number1", "just a number", 30, 200, 30, 20, -4, 122, 3);

    // choice --> args :  oscaddress, label, x, y, width, height, choices
    server.doChoice("/choice1", "just a choice menu", 270, 220, 60, 30, "['one', 'two', 'three']");

    // radio box --> args :  oscaddress, label, x, y, width, height, choices
    server.doRadioBox("/radio1", "just a radio box", 270, 100, 120, 90, "['hey', 'ho', 'lets go']", 1);
}





///////// widgets Listeners
class Toggle1Listener extends OscListener{
    fun void action(){ <<<"chuck: toogle1", e.getInt()>>>; }
}
class Button1Listener extends OscListener{
    fun void action(){ <<<"chuck: button1", e.getInt()>>>; }
}
class Slider1Listener extends OscListener{
    fun void action(){ <<<"chuck: slider1", e.getInt()>>>; }
}
class Slider2Listener extends OscListener{
    fun void action(){ <<<"chuck: slider2", e.getInt()>>>; }
}
class Choice1Listener extends OscListener{
    fun void action(){ <<<"chuck: choice1", e.getString()>>>; }
}
class Radio1Listener extends OscListener{
    fun void action(){ <<<"chuck: radio1", e.getString()>>>; }
}
class Num1Listener extends OscListener{
    fun void action(){ <<<"chuck: num1", e.getInt()>>>; }
}


// widgets listeners//
Button1Listener buttonLis; // button
buttonLis.bind( "/button1, i" );
Toggle1Listener toggleLis; // toggle
toggleLis.bind( "/toggle1, i" );
Slider1Listener sl1Lis; // horizonal slider
sl1Lis.bind( "/slider1, i" );
Slider2Listener sl2Lis; //vertical slider
sl2Lis.bind( "/slider2, i" );
Choice1Listener cho1Lis; // choice
cho1Lis.bind( "/choice1, s" );
Radio1Listener radio1Lis; // radio box
radio1Lis.bind( "/radio1, s" );
Num1Listener num1Lis; // number
num1Lis.bind( "/number1, i" );
 


<<<"chuck osc receiver up and runing ...">>>;
1 => int on;

while (on){ // quits when on is false
    0.2::second => now; 
}

<<<"quiting ChucK">>>; //print quit



